      REAL FUNCTION NETLONGF (TA, EA, RATIO, C1, C2, C3)
C     net longwave radiation,  W/m2
C     emissivity of the surface taken as 1.0 to also account for reflected
C     all equations and parameters from Brutsaert (1982)
C     input
C        TA      air temperature, degC
C        EA      vapor pressure, kPa
C        RATIO   ratio of actual to potential solar radiation
C        C1      intercept of actual/potential relation to n/N
C        C2      slope of actual/potential relation to n/N
C        C3      longwave cloud correction coefficient
C     local
C        SIGMA   Stefan-Boltzmann constant, W/(m2/K4)
C        NOVERN  sunshine duration fraction of daylength
C        EFFEM   effective emissivity from clear sky
C        CLDCOR  cloud cover correction to net longwave under clear sky
      IMPLICIT NONE
      REAL TA, EA, RATIO, C1, C2, C3, SIGMA, NOVERN, EFFEM, CLDCOR
      REAL SQRT, EXP
      INTRINSIC SQRT, EXP
C
      SIGMA = 5.67E-08
C     Brutsaert method
      EFFEM = 1.24 * (EA * 10. / (TA + 273.15)) ** (1. / 7.)
C
C     additional methods not used
C     Brunt method
C        BRUNTA  Brunt intercept
C        BRUNTB  Brunt EA coefficient, for kPa (value for mb * 10)
C     BRUNTA = .44
C     BRUNTB = .253
C     EFFEM = BRUNTA + BRUNTB * SQRT(EA)
C     Satterlund method
C     EFFEM = 1.08 * (1 - EXP(-(10. * EA) ** ((TA + 273.15) / 2016.)))
C     Swinbank method
C     EFFEM = .0000092 * (TA + 273.15) ** 2.
C     Idso-Jackson method
C     EFFEM = 1. - .261 * EXP(-.000777 * TA ** 2.)
C
      NOVERN = (RATIO - C1) / C2
      IF (NOVERN .GT. 1.) NOVERN = 1.
      IF (NOVERN .LT. 0.) NOVERN = 0.
      CLDCOR = C3 + (1. - C3) * NOVERN
      NETLONGF = (EFFEM - 1.) * CLDCOR * SIGMA * (TA + 273.15) ** 4

c      print*, 'here 1b', TA, EA, SIGMA, EFFEM
c      print*, 'here 2b', RATIO, C1, C2, C3
c      print*, 'here 3b',  NOVERN, CLDCOR, NETLONGF


      END
