      REAL FUNCTION PMDAYF(DOY,LATD,LONGD,iveg,LAI,SNOW,FL,UW,
     $                      RADNET,TA,EA,SHEAT,TMAX,TMIN,iflag)
C     daily Penman-Monteith PE in mm for day
C     local
C        ALBEDO  albedo
C        DAYLEN  daylength in fraction of day
C        DISP    zero-plane displacement, m
C        DISPC   zero-plane displacement for closed canopy, m
C        I0HDAY  daily potential insolation on horizontal, MJ/m2
C        LAI     projected leaf area index
C        LAT     latitude, radians, south negative
C        SAI     projected stem area index
C        UA      average wind speed for the day, m/s
C        ZA      reference height, m
C        Z0      surface roughness parameter, m
C        Z0C     surface roughness parameter for closed canopy, m
C        SOLNET  average net solar radiation for daytime, W/m2
C        LNGNET  average net longwave radiation for day,-* W/m2
C        AA      available energy, W/m2
C        DD      vapor pressure deficit, kPa
C        RC      canopy or surface resistance, s/m
C        RA      aerodynamic resistance, s/m
C        LE      latent heat, W/m2
C     constant
C        IGRATE  integrates W/m2 over 1 d to MJ/m2, 86400 s/d * 
C                                                   .000001 MJ/J * 1 d
C        ETOM    converts MJ/m2 to mm of water
      include "HLCOMM_4arctic.INC"

      INTEGER DOY,iveg,iflag
      REAL LATD, LONGD, RADNET, test, testval
      REAL ALBEDO, DAYLEN, DISP, DISPC, I0HDAY, LAI, LAT, SAI
      REAL UA, ZA, Z0, Z0C
      REAL ES, DELTA, SOLNET, LNGNET, AA, DD, RC, RA, LE
      REAL IGRATE, ETOM
      REAL NETLONGF, SRSCF, PMF, WNDADJF
      REAL LOG
C      INTRINSIC LOG - FBM SGI did not like it
      EXTERNAL NETLONGF, ESATF, SRSCF, PMF, WNDADJF, SUNDHF, ROUGHF
      include "HLPECOMM.INC"

      IGRATE = .0864
      ETOM = .4085

c  Call subroutine to access vegetation-specific parameters
      call getvegvals(iveg,ALB,ALBEDO,CR,HEIGHT,LPMAX,GLMAX,R5,CVPD,
     $               LWIDTH,Z0G,RSS,FETCH,ZW,Z0W)

C      print*, iflag

      if (iflag .eq. 1) then
         write(97,*) 'iveg = ', iveg
         write(97,*) 'ALB = ', ALB
         write(97,*) 'Albedo = ', ALBEDO
         write(97,*) 'CR = ', CR
         write(97,*) 'HEIGHT = ', HEIGHT
         write(97,*) 'GLMAX = ', GLMAX
         write(97,*) 'R5 = ', CVPD
         write(97,*) 'LWIDTH = ',LWIDTH 
         write(97,*) 'Z0G = ', Z0G
         write(97,*) 'FETCH = ', FETCH
         write(97,*) 'ZW = ', ZW
         write(97,*) 'Z0W ', Z0W
         write(97,*) 'DOY = ', DOY
         write(97,*) 'LATD = ', LATD
         write(97,*) 'LONGD = ', LONGD
         write(97,*) 'LAI = ', LAI
         write(97,*) 'SNOW = ', SNOW 
         write(97,*) 'FL = ', FL
         write(97,*) 'UW = ', LATD
         write(97,*) 'RADNET = ', RADNET
         write(97,*) 'TA = ', TA
         write(97,*) 'EA = ', EA
         write(97,*) 'RADNET = ', RADNET
         write(97,*) 'SHEAT = ', SHEAT
      endif   
C
      
      LAT = LATD / 57.296
      CALL SUNDHF(LAT, DOY, DAYLEN, I0HDAY)
c      if (iflag .eq. 1) print*, LAT, DOY, DAYLEN, I0HDAY
      IF (SNOW .GT. 0) THEN
         ALBEDO = ALBSN
      ELSE
         ALBEDO = ALB
      END IF
      IF (R5 .GT. RM / 2) THEN
         PRINT*, 'R5 must be < RM/2'
         STOP
      END IF
      IF (LPMAX .GT. LPC) THEN
         SAI = CS * HEIGHT
      ELSE
c         SAI = (LPMAX / LPC) * CS * HEIGHT              ! original
         SAI = real(LPMAX) / real(LPC) * CS * HEIGHT      ! fix
      END IF

c      print*, DAYLEN, I0HDAY, snow, ALBEDO
c      print*, lpmax, lpc, CS, HEIGHT, sai
      
c      LAI = FL * LPMAX
c      LAI = MAX(0.001, LAI)

c  LAI is read in input from calling program, instead of calculating it
c  from LPMAX.


      CALL ROUGHF(CZS, CZR, HS, HR, LPC, CS, LAI, SAI, HEIGHT, Z0G,
     *            Z0, DISP, Z0C, DISPC)
c      print*, CZS, CZR, HS, HR, LPC, CS, LAI, SAI, HEIGHT, Z0G 
c      print*, Z0, DISP, Z0C, DISPC
      ZA = HEIGHT + ZMINH
      IF (UW .LT. 0.2) UW = 0.2
      UA = UW * WNDADJF(ZA, DISP, Z0, FETCH, ZW, Z0W)

c  Call subroutine to get solrad from netrad
      call net2solrad(RADNET,TA,EA,ALBEDO,C1,C2,C3,I0HDAY,SOLRAD,iflag)
c      write(96,*) RADNET,TA,EA,ALBEDO,C1,C2,C3,I0HDAY,SOLRAD
cC
c      SOLNET = (1 - ALBEDO) * SOLRAD / IGRATE
c      LNGNET = NETLONGF(TA, EA, SOLRAD / I0HDAY, C1, C2, C3)
c      AA = SOLNET + LNGNET - SHEAT
c      write(96,*) TA, EA, SOLRAD / I0HDAY, C1, C2, C3, LNGNET

c  Using input net radiation.
      AA = RADNET

c      print*, ALBEDO, SOLRAD, IGRATE, LNGNET, TA, EA, SOLNET
      
      CALL ESATF(TA, ES, DELTA)
      DD = ES - EA

      if (iflag .eq. 1) then
         write(97,*) 'TA = ', TA
         write(97,*) 'ES = ', ES
         write(97,*) 'DELTA = ', DELTA 
         write(97,*) 'RADNET = ', RADNET
         write(97,*) 'SOLRAD = ', SOLRAD
      endif   

c      print*, 'NETRAD = ', NETRAD
c      print*, 'TA = ', TA 
c      print*, 'EA = ', EA
c      print*, 'ALBEDO = ', ALBEDO 
c      write(99,*) 'SOLRAD (W/m2) = ', SOLRAD
c      testval = SOLRAD / I0HDAY
c      test = NETLONGF(TA, EA, testval, C1, C2, C3)
c      print*, test
      
c      if (iflag .eq. 1) SOLRAD = 177.23
      RC = SRSCF(SOLRAD, TMIN, DD, LAI, SAI, R5, CVPD, RM,
     *           CR, GLMAX, GLMIN)
      RA = (LOG ((ZA - DISP) / Z0)) ** 2 / (0.16 * UA)

      if (iflag .eq. 1) then
         write(97,*) 'SOLRAD = ', SOLRAD
         write(97,*) 'TMIN = ', TMIN
         write(97,*) 'LAI = ', LAI
         write(97,*) 'SAI = ', SAI
         write(97,*) 'GLMIN =', GLMIN
         write(97,*) 'GLMAX = ', GLMAX
         write(97,*) 'R5 = ', R5
         write(97,*) 'CVPD = ', CVPD
         write(97,*) 'RM = ', RM
         write(97,*) 'CR = ', CR
         write(97,*) 'RC = ', RC
         write(97,*) ' ' 
      endif

      LE = PMF(AA, DD, DELTA, RA, RC)
      PMDAYF = ETOM * IGRATE * LE

      if (iflag .eq. 1) then
c         write(98,'(6f10.2)') AA, DD, DELTA, RA, RC, PMDAYF
         write(97,*) 'ZA = ',ZA 
         write(97,*) 'DISP = ', DISP
         write(97,*) 'Z0 = ', Z0
         write(97,*) 'UA = ', UA
         write(97,*) 'RA = ', RA
         write(97,*) 'LE = ', LE
       endif


       if (iflag .eq. 1) then
          write(97,*) 'DELTA = ', DELTA
          write(97,*) 'AA = ', aa
          write(97,*) 'DD = ', DD    
          write(97,*) 'Es = ', es
          write(97,*) 'Ea = ', ea
          write(97,*) 'RA = ', RA
          write(97,*) 'RC = ', RC
          write(97,*) 'LE = ', LE
          write(97,*) 'PMDAYF = ', PMDAYF
          write(97,*) ' '
       endif


       END
