      SUBROUTINE ROUGHF (CZS, CZR, HS, HR, LPC, CS, LAI, SAI, HEIGHT,
     *                   Z0G, Z0, DISP, Z0C, DISPC)
C     roughness parameter and zero-plane displacement
C     input
C        LAI     projected leaf area index
C        SAI     projected stem area index
C        LPC     projected leaf area index for closed canopy
C        CZS     ratio of roughness to height for smooth closed canopies
C        CZR     ratio of roughness to height for rough closed canopies
C        HR      height above which CZR applies, m
C        HS      height below which CZS applies, m
C        HEIGHT  canopy height, m
C        Z0G     ground surface roughness, m
C     output
C        Z0      roughness parameter, m
C        DISP    zero-plane displacement, m
C        Z0C     roughness length for closed canopy, m
C        DISPC   zero-plane displacement for closed canopy, m
C     local
C        XX, CDRAG
      IMPLICIT NONE
      REAL CS,LAI, SAI, LPC, CZS, CZR, HR, HS, HEIGHT, Z0G
      REAL Z0, DISP, Z0C, DISPC
      REAL XX, CDRAG
      REAL MIN, EXP, LOG
C      INTRINSIC MIN, EXP, LOG - FBM SGI did not like
C
      IF (HEIGHT .GE. HR) THEN
         Z0C = CZR * HEIGHT
      ELSE IF (HEIGHT .LE. HS) THEN
         Z0C = CZS * HEIGHT
      ELSE
         Z0C = CZS * HS + (CZR * HR - CZS * HS) * 
     *         (HEIGHT - HS) / (HR - HS)
      END IF

C     Limit Z0C to Z0G 2001-05-23 Tony and FBM
      IF (Z0C .LT. Z0G) THEN
         Z0C = Z0G
      END IF
     
      DISPC = HEIGHT - Z0C / .3
C     Changed the if statement IF (LAI .GE. LPC) 2001-05-23 Tony and FBM
      IF (LAI + SAI .GE. LPC + CS * HEIGHT) THEN
C       closed canopy
         Z0 = Z0C
         DISP = DISPC
      ELSE
C       sparse canopy
C       from Shuttleworth and Gurney (1990)
         CDRAG = (-1 + EXP(.909 - 3.03 * Z0C / HEIGHT)) ** 4
     *           / (LPC + CS * HEIGHT)
         XX = CDRAG * (LAI + SAI)
         DISP = 1.1 * HEIGHT * LOG(1 + XX ** .25)
         Z0 = MIN(.3 * (HEIGHT - DISP), Z0G + .3 * HEIGHT * XX ** .5)
      END IF
      END
