      SUBROUTINE SUNDHF (zLAT, iDOY, DAYLEN, I0HDAY)
C     daylength, and potential daily solar radiation on a horizontal surface
C     Lee (1978), Swift (1976)
C     inputs
C       LAT      latitude, radians
C       DOY      day of the year
C     outputs
C       DAYLEN   daylength fraction of day, d
C       I0HDAY   daily potential solar radiation (horizontal) MJ/m2
C     local
C       DEC      declination
C       PI       pi
C       ARG, H
C       ISC      solar constant
      IMPLICIT NONE
      INTEGER iDOY
      REAL zLAT, rLAT, DAYLEN, I0HDAY, REALDOY
      REAL DEC, PI, ARG, H, ISC
      REAL ABS, ASIN, SIN, SIGN, TAN, ACOS, COS
      INTRINSIC ABS, ASIN, SIN, SIGN, TAN, ACOS, COS
C
      REALDOY = real(iDOY)
      PI = 3.14159
      ISC = 1367. / (1. - .0167 * COS(.0172 * (REALDOY - 3.))) ** 2
      DEC = ASIN(.39785 * SIN(4.868961 + .017203 * REALDOY +
     *          .033446 * SIN(6.224111 + .017202 * REALDOY)))
C     halfday length
      IF (ABS(zLAT) .GE. PI / 2.) then
         rLAT = SIGN(PI / 2. - .01, zLAT)
      ELSE
         rLAT = zLAT
      ENDIF   
      ARG = -TAN(DEC) * TAN(rLAT)
      IF (ARG .GE. 1.) THEN
C       sun stays below horizon
         H = 0
      ELSE IF (ARG .LT. -1.) THEN
C       sun stays above horizon
         H = PI
      ELSE
         H = ACOS(ARG)
      END IF
      DAYLEN = H / PI
C     daily potential from Sellers (1965)
      I0HDAY = 0.000001 * ISC * (86400. / PI) *
     *       (H * SIN(rLAT) * SIN(DEC) + COS(rLAT) * COS(DEC) * SIN(H))

      END

