C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: check  *****             ##
C#    Subroutine checks if input value satisfies criteria             ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************

c     
c  Subroutine checks to see if an input value satisfies an input criteria
c  to within a specified input tolerance.
c  Inputs:
c     value - the value to be checked (double precision real variable)
c     precision - a specified allowable tolerance (double precision real)
c     critval - the threshold (double precision real)
c     criteria - a character*2 string (eg., lt, gt, eq)
c
      subroutine check(value,precision,critval,criteria)

      implicit none
      real*8 value,precision,critval,threshold
      character*2 criteria

      if (critval .eq. 0.0d0 .and. criteria .eq. 'lt') then
         threshold = critval - precision
         if (value .lt. threshold) stop 'Less than zero check violated'
      else if (critval .eq. 0.0d0 .and. criteria .eq. 'gt') then
         if (value .gt. threshold)stop'Greater than zero check violated'
      else if (critval .eq. 1.0d0 .and. criteria .eq. 'gt') then
         threshold = critval + precision
         if (value .gt. threshold) stop'Greater than one check violated'
      else if (critval .eq. 1.0d0 .and. criteria .eq. 'lt') then
         threshold = critval - precision
         if (value .lt. threshold) stop 'Less than one check violated'
      else if (critval .eq. 0.0d0 .and. criteria .eq. 'eq') then
         threshold = critval + precision
         if (value .gt. threshold)stop'Not equal to zero check violated'
         threshold = critval - precision
         if (value .lt. threshold)stop'Not equal to zero check violated'
      endif

      return
      end
      
