C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: daylength  *****         ##
C#  Subroutine gets daylength as function of latitude and day of year ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
      
      subroutine daylength(r_lat2, dayofyr, day_lngth)

      implicit none
      real*8 r_lat2, radlat, pi, arg, h, dayofyr, day_lngth, dec, dtor
      parameter (pi=3.14159d0)


      dtor = pi / 180.0d0    ! parameter to convert lat/lon from degs to rads 
      radlat = r_lat2 * dtor

      dec = asin(0.39785d0 * sin(4.868961d0 + 0.017203d0 * dayofyr
     $     + 0.033446d0 * sin(6.224111d0 + 0.017202d0 * dayofyr)))
 
      if (abs(radlat) .ge. pi/2.0d0) then     
         if (radlat .ge. 0.0d0) then
            radlat = abs(pi / 2.0d0 - 0.01d0)
         else
            radlat = (-1) * (abs(pi / 2.0d0 - 0.01d0))
         endif
      endif 
 
      arg = -tan(dec) * tan(radlat)
 
      if (arg .ge. 1.0d0) then            ! sun stays below horizon 
         h = 0.0d0
      else if (arg .lt. -1.0d0) then      ! sun stays above horizon
         h = pi
      else
         h = acos(arg)
      endif    
 
      day_lngth = h / pi

      return
      end
