!#######################################################################
!#                                                                    ##
!#               PPPP   W           W  BBBBBB   M       M             ##
!#               P   P   W         W   B    B   MM     MM             ##
!#               PPPP     W   W   W    BBBBBB   M M   M M             ##
!#               P         W W W W     B    B   M  M M  M             ##
!#               P          W   W      BBBBBB   M   M   M             ##
!#                                                                    ##
!#                                                                    ##
!#                    PERMAFROST WATER BALANCE MODEL                  ##
!#                     +++++   Version 4.0   +++++                    ##
!# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
!#                                                                    ##
!#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
!#                                                                    ##
!#                  https://blogs.umass.edu/csrc/pwbm                 ##
!#                                                                    ##
!# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
!#                                                                    ##
!#              *****  PWBM (v4) subroutine: docleaching  *****       ##
!#    Subroutine handles variables for model DOC leaching             ##
!#                                                                    ##
!#                                                                    ##
!#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
!#                                                                    ##
!***********************************************************************

  subroutine doc4surface(soilT,soilDOC,ksnowwatflag,kspringflag,&
       ksurfwatcount,watertoday,surfpoolrunoff,densityC,production_k,&
       decomp_k,DOCsurface,iwrite)

  implicit none

  ! LOCAL VARIABLES:
  integer i,lx,sx
  parameter(lx=23,sx=-5)
  real*8 :: a,b,excess4layer,soilCdensity,exponent
  real*8 :: factor,ratio,productionDOC,decomp,production
  real*8 :: function_temp,function_moist,val_m1,val_mopt,val_msat,bval
  real*8 :: soilCdensity_default,soilCdensity_tmp,diff1,diff2
  real*8 :: production_k_default,production_k_tmp,returnval,scalefactor(5)
  real*8, parameter :: saturationFraction=1.0d0,scalingfactor=5.0d2  
  data scalefactor/6.0d2,7.5d2,9.5d2,9.5d2,8.0d2/   

  ! Input Variables
  !  integer :: isnowclas
  integer iwrite,ksnowwatflag,ksurfwatcount,kspringflag
  real*8 :: soilT(sx:lx),surfpoolrunoff,densityC,decomp_k,production_k
  real*8 :: watertoday(sx:lx),soilDOC(0:lx)
!
! Output Variables... ! soil layer DOC and weighted subsurface DOC 
  real*8 :: DOCsurface 
!-----------------------------------------------------------------------

!   Potential SOC for leaching based on SOC of upper meter  
   soilCdensity = densityC          ! in grams C m-2

!  This is the base value set in initialization file  
  production_k_default = production_k
  soilCdensity_default = soilCdensity

  if (ksurfwatcount == 0 .or. ksurfwatcount > 5 .or. kspringflag == 0) then
     production_k = production_k_default
     soilCdensity = soilCdensity_default
  else if (ksurfwatcount <= 5) then
     production_k = production_k_default * scalefactor(ksurfwatcount)
     soilCdensity = soilCdensity_default * scalefactor(ksurfwatcount)
  endif

  if (ksnowwatflag == 1) then
     production_k = production_k_default * scalefactor(1)
     soilCdensity = soilCdensity_default * scalefactor(1)     
  endif   

!  Influence of soil moisture on decomposition and DOC production.       
  function_moist = saturationFraction ** (1.0d0)

!  Q10 function for influence of temperature. Using uppermost soil layer temp.
  function_temp = 2.0d0 ** (soilT(1)/10.0d0)

!  Estimate DOC production and add to the pool.  
  production = production_k * soilCdensity * function_temp * function_moist
  soilDOC(0) = soilDOC(0) + production

!  Estimate decomposition. Function implicitly includes C sorb to soil.   
!  Subtract decomposition from the storage pool.
  decomp = decomp_k * soilDOC(0) * function_temp * function_moist
  soilDOC(0) = soilDOC(0) - decomp

!  Leached DOC from the soil DOC pool based on surface runoff.
  DOCsurface = soilDOC(0) * surfpoolrunoff
  DOCsurface = min(DOCsurface,soilDOC(0))
  factor = production * 2.0d0
  DOCsurface = min(DOCsurface,factor)   
  soilDOC(0) = soilDOC(0) - DOCsurface
  if (DOCsurface == 0.0d0) DOCsurface = -888.8	
     
end subroutine doc4surface

