!#######################################################################
!#                                                                    ##
!#               PPPP   W           W  BBBBBB   M       M             ##
!#               P   P   W         W   B    B   MM     MM             ##
!#               PPPP     W   W   W    BBBBBB   M M   M M             ##
!#               P         W W W W     B    B   M  M M  M             ##
!#               P          W   W      BBBBBB   M   M   M             ##
!#                                                                    ##
!#                                                                    ##
!#                    PERMAFROST WATER BALANCE MODEL                  ##
!#                     +++++   Version 4.0   +++++                    ##
!# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
!#                                                                    ##
!#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
!#                                                                    ##
!#                  https://blogs.umass.edu/csrc/pwbm                 ##
!#                                                                    ##
!# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
!#                                                                    ##
!#              *****  PWBM (v4) subroutine: docleaching  *****       ##
!#    Subroutine handles variables for model DOC leaching             ##
!#                                                                    ##
!#                                                                    ##
!#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
!#                                                                    ##
!***********************************************************************

  subroutine docleaching(iWinter,soil_xnodes,peatfrac,soilT,water4layer,&
     capacity4layer,baseflow_array,densityC1,densityC2,densityC3,&
       soilDOC,production_k,decomp_k,doc_layer,DOCsubsurface,iwrite)

  implicit none

  ! LOCAL VARIABLES:
  integer i,lx,sx
  parameter(lx=23,sx=-5)
  real*8 :: function_temp,function_moist,excess4layer,fracC4layer,soilCdensity
  real*8 :: ratio,productionDOC,rsumDOC,rsumweight,doc_leached
  real*8 :: val_m1,val_mopt,val_msat,bval,val_moist,saturationFraction
  real*8 :: porosity4layer(lx),dzmm(lx),production,decomp,soilDOC2write
  real*8 :: argument,argument2,decomp2

  ! Input Variables
  integer iwrite,iWinter
  real*8 :: soil_xnodes(sx:lx),soilT(sx:lx),peatfrac(lx)
  real*8 :: capacity4layer(sx:lx),water4layer(sx:lx)
  real*8 :: densityC1,densityC2,densityC3
  real*8 :: production_k,decomp_k,baseflow_array(lx),soilDOC(0:lx)
!
! Output Variables... ! soil layer DOC and weighted subsurface DOC 
  real*8 :: doc_layer(0:lx),DOCsubsurface  
!
!-----------------------------------------------------------------------

  if (densityC1 <= 0.0d0) densityC1 = 10.0d0
  if (densityC2 <= 0.0d0) densityC2 = 10.0d0   ! these in hg C / m2
  if (densityC3 <= 0.0d0) densityC3 = 10.0d0

  call compute_dzmm(soil_xnodes*1.0d3, lx, dzmm)
  
  rsumDOC = 0.0d0
  rsumweight = 0.0d0

  do i = 1, lx    

     doc_layer(i) = 0.0d0   ! set to zero outside baseflow if check

!     Converting hg C m-2 to g C m-2.      
     if (soil_xnodes(i) .ge. 0.0d0 .and. soil_xnodes(i)  &
          .le. 1.0d0) then
        soilCdensity = (densityC1 * 1.0d2) 
     else if (soil_xnodes(i) .gt. 1.0d0 .and. soil_xnodes(i) &
          .le. 2.0d0) then
        soilCdensity = (densityC2 * 1.0d2) 
     else if (soil_xnodes(i) .gt. 2.0d0 .and. soil_xnodes(i) &
          .le. 3.0d0) then
        soilCdensity = (densityC3 * 1.0d2) 
     else if (soil_xnodes(i) .gt. 3.0d0) then
        soilCdensity = (densityC3 * 1.0d2) 
     endif                            
     if (soilCdensity .eq. 0.0d0) soilCdensity = 2.0d2 

     val_m1 = 0.2d0
     val_mopt = 60.0d0
     val_msat = 0.6d0
     if (capacity4layer(i) > 0.0d0) then
        saturationFraction = water4layer(i) / capacity4layer(i)  
     else
        saturationFraction = 0.0d0
     endif   

!  Here is Q10 function. Different in warm and cold season. 
     function_temp = 2.0d0 ** (soilT(i)/10.0d0)

!  Using % soil moisture for moisture influence. Exponent 1 (see Wen et al.)
     if (saturationFraction .gt. 0.0d0) then
        function_moist = saturationFraction ** (1.0d0)
     else
        function_moist = 0.1d0 ** (1.0d0)   
     endif
!  Argument for Q10 function. This is equation 1.10b in Raich et al. 1991.
     argument = function_temp * function_moist

     production = production_k * soilCdensity * argument    

!     soilDOC(i) = soilDOC(i) + (production - decomp)
     soilDOC(i) = soilDOC(i) + production

     decomp = decomp_k * soilDOC(i) * argument
     soilDOC(i) = soilDOC(i) - decomp

     if (baseflow_array(i) > 0.0d0) then

!  DOCsurface = production * surfpoolrunoff
        doc_layer(i) = soilDOC(i) * baseflow_array(i) 
        doc_layer(i) = min(doc_layer(i),soilDOC(i))
	soilDOC2write = soilDOC(i)
        soilDOC(i) = soilDOC(i) - doc_layer(i) 
     else
        doc_layer(i) = 0.0d0
        soilDOC2write = soilDOC(i)
     endif
     rsumDOC = rsumDOC + doc_layer(i)

  enddo

  if (rsumDOC > 0.0d0) then
     DOCsubsurface = rsumDOC
  else
     DOCsubsurface = 0.0d0
  endif   
     
end subroutine docleaching
