C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: getvegvals  *****        ##
C#    Subroutine maps TEMVEG classes into WBM vegetation class        ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
      
      subroutine getvegvals(iveg,ALB,ALBEDO,CR,HEIGHT,LPMAX,GLMAX,R5,
     $               CVPD,LWIDTH,Z0G,RSS,FETCH,ZW,Z0W)

      integer iveg,nveg,inewveg
      parameter (nveg=9)

      real ALB,ALBEDO,CR,HEIGHT,LPMAX,GLMAX,R5,CVPD,LWIDTH,Z0G,RSS
      real FETCH,ZW,Z0W

      real ALB1(nveg),ALBEDO1(nveg),CR1(nveg),HEIGHT1(nveg),LPMAX1(nveg)
      real GLMAX1(nveg),R51(nveg),CVPD1(nveg),LWIDTH1(nveg),Z0G1(nveg)
      real RSS1(nveg),FETCH1(nveg),ZW1(nveg),Z0W1(nveg)

      data ALB1 /0.14, 0.18, 0.18, 0.2, 0.2, 0.22, 0.26, 0.1, 0.17/
      data ALBEDO1 /0.14, 0.23, 0.35, 0.5, 0.5, 0.5, 0.5, 0.5, 0.32/
      data CR1 /0.5, 0.6, 0.6, 0.7, 0.7, 0.7, 0.7, 0.01, 0.6/
      data HEIGHT1 /25., 25., 8., 0.5, 0.3, 0.3, 0.1, 0.01, 12.3/
      data LPMAX1 /6., 6., 3., 3., 4., 3., 1., 0.00001, 5.0/
      data GLMAX1 /0.0053, 0.0053, 0.0053, 0.008, 0.0066, 0.011, 0.005,
     $             0.001, 0.01/
      data R51 /100., 100., 100., 100., 100., 100., 100., 10., 100./
      data CVPD1 /2., 2., 2., 2., 2., 2., 2., 0.1, 2./
      data LWIDTH1 /0.0, 0.1, 0.03, 0.01, 0.01, 0.1, 0.02, 0.001, 
     $   0.005/
      data Z0G1 /0.02, 0.02, 0.02, 0.01, 0.01, 0.005, 0.001, 0.001,
     $   0.015/
      data RSS1 /500., 500., 500., 500., 500., 500., 500., 500., 500./
      data FETCH1/5000., 5000., 5000., 5000., 5000., 5000., 5000., 
     $   5000., 5000./
      data ZW1 /10., 10., 10., 10., 10., 10., 10., 10., 10./
      data Z0W1 /0.005, 0.005, 0.005, 0.005, 0.005, 0.005, 0.005, 0.005,
     $   0.005/

c***************************************************************************
c  Map TEMVEG classes into vegetation classes from A. Federer/WBM for 
c  WBM parameters defined here.

      if (iveg.eq.9.or.iveg.eq.-9999.) then
         print*, 'Missing vegetation value'
         stop 'Missing vegetation value'
      endif

      if (iveg .eq. 1 .or. iveg .eq. 2) inewveg = 8
      if (iveg .eq. 3) inewveg = 5
      if (iveg .eq. 4) inewveg = 9
      if (iveg .eq. 5) inewveg = 1
      if (iveg .eq. 6) inewveg = 4
      if (iveg .eq. 7) inewveg = 5
      if (iveg .eq. 8) inewveg = 2
      if (iveg .eq. 10) inewveg = 8

      if (inewveg .lt. 1 .or. inewveg .gt. 9) then
         print*, 'inewveg out of range == ', inewveg
         stop 'inewveg out of range == '
      endif
      
c Set scaler values (to be passed as returned values) from the data list above.
      ALB = ALB1(inewveg)
      ALBEDO = ALBEDO1(inewveg)
      CR = CR1(inewveg)
      HEIGHT = HEIGHT1(inewveg)
      LPMAX = LPMAX1(inewveg)
      GLMAX = GLMAX1(inewveg)
      R5 = R51(inewveg)
      CVPD = CVPD1(inewveg)
      LWIDTH = LWIDTH1(inewveg)
      Z0G = Z0G1(inewveg)
      RSS = RSS1(inewveg)
      FETCH = FETCH1(inewveg)
      ZW = ZW1(inewveg)
      Z0W = Z0W1(inewveg)

      return
      end
