C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: hamon  *****             ##
C#    Subroutine calculates PET by Hamon routine                      ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
      
      subroutine hamon(r_lat, idayofyr, temp, pet)

      implicit none
      integer i, idayofyr
      real r_lat
      real*8 r_lat1, temp, pet, es, day, daylngth, rhosat

c  Calculate stauration vapor pressure
c  Input temperature is in degrees C and vapor pressure output in kPa
      if (temp .lt. 0.0d0) then 
         es = exp( temp*21.87456d0 / (temp + 265.5d0)) 
     $        * 0.61078d0 
      else
         es = exp(temp * 17.26939d0 / (temp + 237.3d0)) 
     $        * 0.61078d0
      endif

c  Call routine to compute length of day (sun above horizon).  Number
c  returned is in interval [0,1]
      day = dble(idayofyr)      ! this is the day of year (1 to 365)
      r_lat1 = dble(r_lat)
      call daylength(r_lat1, day, daylngth) 

c  Calculate density
      rhosat = es * 2.167d0 / (temp + 273.15d0)

c  Compute the potential evaporation
      pet = daylngth * 330.2d0 * rhosat

c      write(*,'(7f10.5)') temp, es, day, r_lat, rhosat, daylngth, pet          
  
      return
      end
