C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: lookup_v4  *****         ##
C#    Subroutine returns soil parameters based on soil class          ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
      
      subroutine lookup_v3(isoiltype,ilayers,peat_fraction,
     $ SoilPorosity,FieldCapacity,WiltingPoint,BulkDensity,ThermCondDry,
     $ HeatCapacityDry,FreezingPD,SoilCleaness,porositycombined,
     $ fcapacitycombined,hksatcombined,psisatcombined,beta_combined)

      implicit none
      integer i,isoiltype,ilayers
      real*8 SoilPorosity,FieldCapacity,WiltingPoint,BulkDensity
      real*8 ThermCondDry, HeatCapacityDry,peatporosity
      real*8 FreezingPD, SoilCleaness,peat_fraction(ilayers)
      real*8 porositycombined(ilayers),fcapacitycombined(ilayers)
      real*8 peatFC,hksat,psisat,psisatpeat,hksatcombined(ilayers)
      real*8 psisatcombined(ilayers),hksatpeat,beta,peatbeta
      real*8 beta_combined(ilayers)
      parameter(peatporosity = 0.9d0,peatFC=0.65d0,psisatpeat=-1.2d2)
      parameter(hksatpeat=2.0d-2,peatbeta=2.7d0)

      if (isoiltype .eq. 1) then                     ! sand, 3% clay 
c         SoilPorosity    =  0.25d0
         SoilPorosity    =  0.30d0
         FieldCapacity   =  0.05d0
         WiltingPoint    =  0.04d0
         BulkDensity     =  1.60d3
           ThermCondDry    =  0.5d0   !was 0.6d0     ! in J/m/s/degC		
	   HeatCapacityDry =  1.50d0                   ! in Mega J/m/degC
	   FreezingPD	   = -0.03d0
	   SoilCleaness    = -0.70d0
           hksat = 2.3558d-2
           psisat = -4.729d1
           beta = 3.4d0
      else if (isoiltype .eq. 2 .or. isoiltype .eq. 5 .or. isoiltype
     $        .eq. 7 .or. isoiltype .eq. 8) then     
         SoilPorosity    =  0.35d0           ! loam, 20:40:40 clay,silt,sand
         FieldCapacity   =  0.24d0    ! was 0.24d0
         WiltingPoint    =  0.09d0
         BulkDensity     =  1.44d3
           ThermCondDry    =  0.45d0   ! was 0.49d0
	   HeatCapacityDry =  1.50d0	
	   FreezingPD	   = -0.03d0
	   SoilCleaness    = -0.55d0
           hksat = 4.192d-2
           psisat = -2.0734d2
           beta = 6.1d0
      else if (isoiltype .eq. 3) then                ! clay, 58% clay
         SoilPorosity    =  0.45d0
         FieldCapacity   =  0.35d0
         WiltingPoint    =  0.22d0
         BulkDensity     =  1.21d3
           ThermCondDry    =  0.385d0 !0.35d0
	   HeatCapacityDry =  1.50d0		
	   FreezingPD	   = -0.03d0
	   SoilCleaness    = -0.40d0
           hksat = 2.0d-2
           psisat = -3.9066d2
           beta = 12.1d0
      else if (isoiltype .eq. 4) then                ! sandy loam, 10% clay
         SoilPorosity    =  0.30d0
         FieldCapacity   =  0.14d0
         WiltingPoint    =  0.05d0
         BulkDensity     =  1.50d3
           ThermCondDry    =  0.55d0 !0.50d0
	   HeatCapacityDry =  1.50d0		
	   FreezingPD	   = -0.03d0
	   SoilCleaness    = -0.62d0
           hksat = 7.111d-2
           psisat = -1.3188d2
           beta = 4.5d0
      else if (isoiltype .eq. 6) then                ! clay loam, 33% clay
         SoilPorosity    =  0.40d0
         FieldCapacity   =  0.32d0
         WiltingPoint    =  0.17d0
         BulkDensity     =  1.35d3
           ThermCondDry    =  0.44d0 !0.40d0
	   HeatCapacityDry =  1.50d0		
	   FreezingPD	   = -0.03d0
	   SoilCleaness    = -0.48d0
           hksat = 2.845d-2
           psisat = -2.8893d2
           beta = 8.2d0
      else if (isoiltype .eq. -99) then       ! Open Water
         SoilPorosity    =  0.39d0            ! modified to assign non-zero
         FieldCapacity   =  0.05d0            ! values for this class
         WiltingPoint    =  0.04d0
         BulkDensity     =  1.60d3            ! plan to treat -99 types in 
         ThermCondDry    =  0.70d-1*4.2d0     !to joules
	   HeatCapacityDry =  1.70d0          ! in Mega J/m/degC
	   FreezingPD	   = -0.03d0
	   SoilCleaness    = -1.0d0
           hksat = 2.845d-2
           psisat = -2.8893d2
           beta = 5.0d0
      else
         print*, 'ERROR: soil texture should not be ', isoiltype
         stop
      endif

C/* FTDJN	
	ThermCondDry=ThermCondDry*6.0d0
	HeatCapacityDry=HeatCapacityDry*2.0d0
C*/ FTDJN

      do i = 1, ilayers
         porositycombined(i) = ((1.0 - peat_fraction(i)) * 
     $       SoilPorosity)  +  (peat_fraction(i) * peatporosity)
         fcapacitycombined(i) = ((1.0 - peat_fraction(i)) * 
     $       FieldCapacity)  +  (peat_fraction(i) * peatFC)
         hksatcombined(i) = ((1.0 - peat_fraction(i)) * 
     $       hksatpeat)  +  (peat_fraction(i) * hksatpeat)
         psisatcombined(i) = ((1.0 - peat_fraction(i)) * 
     $       psisat)  +  (peat_fraction(i) * psisatpeat)
         beta_combined(i) =  ((1.0 - peat_fraction(i)) * 
     $       beta) +  (peat_fraction(i) * peatbeta)
c        write(88,*)i,peat_fraction(i),SoilPorosity,porositycombined(i),
c     $            peatFC,FieldCapacity,fcapacitycombined(i)
      enddo

      return
      end
