C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: netrad2solrad  *****     ##
C#    Subroutine calculates solar radiation from net radiation        ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
      
      subroutine net2solrad(RADnet,TA,EA,ALBEDO,C1,C2,C3,RI0HDAY,SOLRAD,
     $      iflag)

c  Get solar radiation (SOLRAD, W/m2) from net radiation
c  (NETRAD, W/m2). SOLRAD is needed for the light dependence of the canopy
c  resistance.

c  C1, C2, and C3 are parameters.
c  Values should be 0.25, 0.5, 0.2 respectively.
c  I0HDAY is required from the call to SUNDHF.
c  NETRAD is the input.

      RI0HDAY2 = RI0HDAY * (1./0.0864)     ! convert from MJ/m2 to W/m2

      CB = (1. - C3) / C2
      CA = (C3 * (C1+C2) - C1) / C2
      SIGMA = 5.67E-8                          !W/(m2 K4)
      EFFEM = 1.24 * (10. * EA / ( TA + 273.15)) ** (1./7.)
      RNETLONGCLEAR = (EFFEM - 1.) * SIGMA * (TA + 273.15) ** 4.
      SOLRAD = (RADnet - CA * RNETLONGCLEAR) / 
     $               ((1. - ALBEDO) + CB * RNETLONGCLEAR / RI0HDAY2)


      return
      end
