!#######################################################################
!#                                                                    ##
!#               PPPP   W           W  BBBBBB   M       M             ##
!#               P   P   W         W   B    B   MM     MM             ##
!#               PPPP     W   W   W    BBBBBB   M M   M M             ##
!#               P         W W W W     B    B   M  M M  M             ##
!#               P          W   W      BBBBBB   M   M   M             ##
!#                                                                    ##
!#                                                                    ##
!#                    PERMAFROST WATER BALANCE MODEL                  ##
!#                     +++++   Version 4.0   +++++                    ##
!# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
!#                                                                    ##
!#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
!#                                                                    ##
!#                  https://blogs.umass.edu/csrc/pwbm                 ##
!#                                                                    ##
!# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
!#                                                                    ##
!#              *****  PWBM (v4) subroutine: snowdensity  *****       ##
!#    Subroutine handles variables for model snow density             ##
!#                                                                    ##
!#                                                                    ##
!#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
!#                                                                    ##
!***********************************************************************

  subroutine snowdepthDensity(isnowclass,snowdp,layers_snow,&
        dz,z,zi,frac_bottom)

  implicit none  
  integer, parameter :: nlevsno=5 
!
! Input Variables
  integer :: isnowclass  ! snow class[eg. tundra, tiaga,...] (5 Sturm classes)
  real*8 :: snowdp     ! snow depth (m)
!
! Output Variables
  integer :: layers_snow   ! number of snow layers
  real*8 :: z(-4:0)        ! layer node depth (m) over snow only
  real*8 :: dz(-4:0)       ! layer thickness depth (m) over snow only
  real*8 :: zi(-5:0)       ! interface depth (m) over snow only
  real*8 :: frac_bottom

!
! LOCAL VARIABLES:
  integer :: c,l,j      !indices
  real*8 :: density_bottom,depth_half,depth_slope,density_top
  real*8 :: depth_min,depth_max,frac_bot_max
!-----------------------------------------------------------------------
!  print*, 'Snowclass ', isnowclass
  if (isnowclass == 1) then
     density_top = 450.0d0
     density_bottom = 150.0d0
     depth_max = 0.1d0
     depth_min = 0.0d0
     frac_bot_max = 0.3d0
  else if (isnowclass == 2) then
     density_top = 300.0d0
     density_bottom = 150.0d0
     depth_max = 0.7d0
     depth_min = 0.0d0
     frac_bot_max = 0.7d0
  else if (isnowclass == 3) then
     density_top = 200.0d0
     density_bottom = 200.0d0
     depth_max = 0.8d0
     depth_min = 0.2d0
     frac_bot_max = 0.8d0
  endif
! Initialize snow levels and interfaces (lake and non-lake points)

  dz(-nlevsno+1: 0) = 1.0d8
  z (-nlevsno+1: 0) = 1.0d8
  zi(-nlevsno  :-1) = 1.0d8

! Determine snow levels and interfaces for non-lake points

  if (snowdp < 0.01) then
     layers_snow = 0
     dz(-nlevsno+1:0) = 0.0d0
     z (-nlevsno+1:0) = 0.0d0
     zi(-nlevsno+0:0) = 0.0d0
  else
     if ((snowdp >= 0.01d0) .and. (snowdp <= 0.03d0)) then
        layers_snow = -1
        dz(0)  = snowdp
     else if ((snowdp > 0.03d0) .and. (snowdp <= 0.04d0)) then
        layers_snow = -2
        dz(-1) = snowdp / 2.0d0
        dz(0) = dz(-1)
     else if ((snowdp > 0.04d0) .and. (snowdp <= 0.07d0)) then
        layers_snow = -2
        dz(-1) = 0.02d0
        dz(0) = snowdp - dz(-1)
     else if ((snowdp > 0.07d0) .and. (snowdp <= 0.12d0)) then
        layers_snow = -3
        dz(-2) = 0.02
        dz(-1) = (snowdp - 0.02d0) / 2.0d0
        dz(0) = dz(-1)
     else if ((snowdp > 0.12d0) .and. (snowdp <= 0.18d0)) then
        layers_snow = -3
        dz(-2) = 0.02d0
        dz(-1) = 0.05
        dz(0) = snowdp - dz(-2) - dz(-1)
     else if ((snowdp > 0.18d0) .and. (snowdp <= 0.29d0)) then
        layers_snow = -4
        dz(-3) = 0.02d0
        dz(-2) = 0.05d0
        dz(-1) = (snowdp - dz(-3) - dz(-2)) / 2.0
        dz(0) = dz(-1)
     else if ((snowdp > 0.29d0) .and. (snowdp <= 0.41d0)) then
        layers_snow = -4
        dz(-3) = 0.02d0
        dz(-2) = 0.05d0
        dz(-1) = 0.11d0
        dz(0) = snowdp - dz(-3) - dz(-2) - dz(-1)
     else if ((snowdp > 0.41d0) .and. (snowdp <= 0.64d0)) then
        layers_snow = -5
        dz(-4) = 0.02d0
        dz(-3) = 0.05d0
        dz(-2) = 0.11d0
        dz(-1) = (snowdp - dz(-4) - dz(-3) - dz(-2)) / 2.0d0
        dz(0) = dz(-1)
     else if (snowdp > 0.64d0) then
        layers_snow = -5
        dz(-4) = 0.02d0
        dz(-3) = 0.05d0
        dz(-2) = 0.11d0
        dz(-1) = 0.23d0
        dz(0)=snowdp-dz(-4)-dz(-3)-dz(-2)-dz(-1)
     endif
  end if

! The node depths, which are located at the midpoint of the snow layers, and 
! the layer interfaces are both referenced from the soil surface and are 
! defined as negative values
  do j = 0, layers_snow+1, -1
     z(j)    = zi(j) - 0.5d0 * dz(j)    ! node depths
     zi(j-1) = zi(j) - dz(j)          ! layer interfaces
  end do

  ! Determine snow levels and interfaces for lake points

  if (snowdp < 0.2d0) then
     frac_bottom = 0.0d0
  else if (snowdp > 0.4d0) then
     frac_bottom = frac_bot_max   
  else
     depth_half = (depth_max + depth_min) / 2.0d0 
     depth_slope = 10.0d0 / (depth_max - depth_min)
     frac_bottom = frac_bot_max / (1.0d0 +  &
                              exp(depth_slope * (depth_half - snowdp)))
  endif
     
end subroutine snowdepthDensity
