C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: snow  *****              ##
C#    Subroutine computes snowpack dynamics                           ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************

      subroutine snow (snowpack,snowwatpack,snowholdfactor,meltwat_fact,
     $    radmelt,temp,crittemp,prcp,pet,icovertype,rLAI,snowpack_new,
     $   snowwat_new,water2soil,param1,param2,snowmelt_direct,snowsubl,
     $    iwrite)  
c    
      implicit none
      integer icovertype,iwrite
      real*8 pr_rain,pr_snow,snowpack,snowwatpack,water2soil
      real*8 meltwat,potentmelt,rLAI,snowsubl,snowpet,snowpackprm
      real*8 snowmelt,crittemp,snowholdfactor,snowdamming,temp,prcp,pet
      real*8 snowpack_new,snowwat_new,meltwat_fact,potentfreeze
      real*8 radmelt,diff,factor,snowadj,totalin,totalout,check
      real*8 snowfreeze,snowmelt_direct,surface_et,param1,param2

      surface_et = 0.0d0
      snowmelt_direct = 0.0d0
      totalin = snowpack + snowwatpack + prcp

c  Calculate moisture flux from atmosphere
c  if cold -> get snow,  if warm -> get rain
      if (temp .lt. crittemp) then
         pr_snow = prcp
         pr_rain = 0.0d0
      else
         pr_snow = 0.0d0
         pr_rain = prcp
      endif

c  Determine flux to the atmosphere from snowpack;  dependent only on
c  existence of snowpack. Using estimated PET from Hamon.
      if (snowpack .gt. 0.0d0) then
         if (icovertype .eq. 4 .or. icovertype .eq. 5 .or. icovertype
     $        .eq. 8) then
            if (rLAI .lt. 1.0d0) factor = param1
            if (rLAI .ge. 1.0d0) factor = param1 + rLAI
            snowsubl = min(factor*pet, snowpack)
         else
            factor = param2
            snowsubl = min(factor*pet, snowpack)
         endif   
      else
         snowsubl = 0.0d0
      endif
      
c  Set new values of snowpack, snowpack water, available water (melt), and
c  surface water ET to previous values. Then make adjustments based on inputs.
      snowpack_new = snowpack
      snowwat_new = snowwatpack
      water2soil = 0.0d0
      surface_et = 0.0d0

c  Calculate change in snowpack from precipitation
c       if cold, add to snowpack
c       if warm and a snowpack exists, add rain to watpack
c       if warm and no snowpack exists, add rain to available water (to soil)
      if (pr_snow .eq. 0.0d0 .and. pr_rain .eq. 0.0d0) goto 200
      if (temp .lt. crittemp) then
         snowpack_new = snowpack_new + pr_snow   ! add snowfall to snowpack
      else
         if (snowpack .gt. 0.0d0) then
            snowwat_new = snowwat_new + pr_rain    ! add rain to snowwater
         else
            water2soil = water2soil + pr_rain  ! add rain to avail water
         endif
      endif
 200  continue

c  Calculate change in snowpack from surface sublimation. 
      if(snowpack_new .gt. 0.0d0) then
         diff = snowpack_new - snowsubl
         if (diff .ge. 0.0d0) then
            snowpack_new = snowpack_new - snowsubl
         else
            snowsubl = snowpack_new
            snowpack_new = 0.0d0
         endif
      endif

c  Calculate change from solar radiation melt of snowpack, if a snowpack 
c  exists.  If the potential radiation melt exceeds the snowpack, melt all
c  snow and make it snowwater.
      if(snowpack_new .gt. 0.0d0) then
         diff = snowpack_new - radmelt
         if (diff .ge. 0.0d0) then
            snowpack_new =  snowpack_new - radmelt
            snowwat_new = snowwat_new + radmelt
         else
            snowwat_new = snowwat_new + snowpack_new
            snowpack_new = 0.0d0
         endif
      endif
      
c  Calculate melt from snowpack and add to watpack   * OR *
c  refreeze some snowwater to solid snow.
      if (temp .ge. 0.0d0) then
         snowdamming = snowadj(icovertype)

         potentmelt = (5.26d0 + (5.1d0 * temp)
     $        + (0.1824d0 * temp * prcp))
 
         snowmelt = min(potentmelt, snowpack_new)
         snowpack_new = snowpack_new - snowmelt
         snowwat_new = snowwat_new + snowmelt
      else
         potentfreeze = 2.63d0 + (2.55d0 * abs(temp))
         snowfreeze = min(potentfreeze, snowwat_new)
         snowpack_new = snowpack_new + snowfreeze
         snowwat_new = snowwat_new - snowfreeze
      endif

c  Compute meltwat from watpack
      meltwat = 0.0d0
      snowpackprm = snowholdfactor * snowpack_new

      if (snowwat_new .gt. snowpackprm) then
         if (meltwat_fact .eq. 0.0d0) then
            write(*,*) 'Error - - Snowwater meltwat factor = 0'
            stop
         endif   
         meltwat = snowwat_new * meltwat_fact

         if (snowpack_new .le. 0.0d0)     ! if no snowpack, meltwat all water
     $               meltwat = snowwat_new
         meltwat = min(meltwat, snowwat_new)   
         snowwat_new = snowwat_new - meltwat
      endif

c  Restrict amount accumulated to 1 m liquid equivalent. These are
c     essentially glaciers. About 40 grid behave this way in WALE region.
      if (snowpack_new + snowwat_new .gt. 1000.0d0) then
         diff = (snowpack_new + snowwat_new) - 1000.0d0
         if (snowpack_new .ge. diff) then
            meltwat = meltwat + diff
            snowpack_new = snowpack_new - diff
         else if (snowwat_new .ge. diff) then
            meltwat = meltwat + diff
            snowwat_new = snowwat_new - diff
         else   
            if (snowpack_new .gt. snowwat_new) then
               meltwat = meltwat + diff   
               snowpack_new = snowpack_new - diff
            else
               meltwat = meltwat + diff   
               snowwat_new = snowwat_new - diff
            endif   
         endif
      endif 

      water2soil = water2soil + meltwat
      snowmelt_direct = max(meltwat-pr_rain,0.0d0)

      totalout = snowpack_new + snowwat_new + water2soil + snowsubl +
     $       surface_et
      check = totalin - totalout

      if (abs(check) .gt. 1.0d-6) then
         write(*,*) 'Water balance not preserved in snow routine'
         write(*,*) 'Check = ', check
         stop
      endif

      return
      end   
