C#######################################################################
C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                *****  PWBM (v4) soil subroutine  *****             ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
      subroutine soilzone(icell,nsimdays,iWinter,soilrootdepth,iwetland,
     $     peatfrac,densityC1,densityC2,densityC3,stemp_4layer,
     $ production_k,decomp_k,ivegclass,fieldcapacity,hk_sat,
     $ psi_sat,beta,wat4layerprev,ice4layerprev,soilwiltingpoint,
     $ wiltingpoint,soil_pet,infil_limit,rootbaseflowfactor,
     $ deepbaseflowfactor,airtemp,soil_availwater,surfConduct,param3,
     $ param4,soilDOC,soilbulkdensity,porosity4layer,
     $ water4layer,ice4layer,
     $sumbaseflowNEW,soiloverlandROnew,sum_transpiration,capacity4layer,
     $ DOCsubsurface,ET_SWITCH,INFILTRATION_SWITCH,iwrite)
 
      implicit none
      integer i,ii,icell,iflag,iwrite,iwetland,ipeatlayers,nsimdays
      real*8 soilrootdepth,soilfieldcapacity,soilwiltingpoint
      real*8 soilwiltingpoint2,infil_limit,rootbaseflowfactor
      real*8 deepbaseflowfactor,airtemp,soil_pet,soil_availwater
      real*8 soil_infiltration,potential_infiltration
      real*8 soil_runoff,extra,root_fieldcapacity,totwatratio
      real*8 dtime,latentheat,vap_press,deepwatvolume,soilbulkdensity
      real*8 soil_transpiration_update
      real*8 surfConduct,surface_conduct_factor,param3,param4

      real*8 dry_peat_thcnd  ![W/m/C]   !Thermal conductivity of dry peat
      real*8 dry_soil_thcnd  ![W/m/C]   !Thermal conductivity of dry soil
      real*8 dry_peat_heatc  ![J/kg/C]  !Heat capacity of dry peat
      real*8 dry_soil_heatc  ![J/kg/C]  !Heat capacity of dry soil

      INCLUDE 'pwbm.soil_temperature.h'

      integer iroot_flagged(sx:lx),itimestep,ivegclass,iWinter
      real*8 porosity4layer(lx),termconduct(sx:lx),soilDOC(0:lx)
      real*8 r_depth,soil_xnodes(sx:lx),wfractionSUM
      real*8 fieldcapacity(lx),field_capacity(lx),wat4layerprev(sx:lx)
      real*8 capacity4layer(sx:lx),beta(lx),infiltrateMAX
      real*8 water4layer(sx:lx),surplus,secperday,base_flow,pondmx
      real*8 wfractionNEW(sx:lx),changesurfacelayer,excess,carbonLeached
      real*8 ice4layerprev(sx:lx),ice4layer(sx:lx),current_capacity
      real*8 soiloverlandROnew,surplus2soil,watmin,DOCmass
      real*8 baseflowfactor,baseflowNEW,deficit,excess2,peatfrac(lx)
      real*8 watersum4layers,water_in_NEW,water_out_NEW,checkNEW
      real*8 sumbaseflowNEW,wiltingpoint,wiltingPT4layer(lx)
      real*8 trans4layer,sum_transpiration,baseflow_array(lx)
      real*8 vol_ice(lx),eff_porosity(lx),vol_liq(lx),icefrac(lx)
      real*8 fracice(lx),hk_sat(lx),psi_sat(lx),hk(lx),den,sdamp
      real*8 dhkdw(lx),s_node,smp(lx),smpmin,dsmpdw(lx),qin,qout
      real*8 num,dqodw1,dqodw2,rmx(lx),amx(lx),bmx(lx),cmx(lx),s1,s2
      real*8 dqidw0,dqidw1,zmm(lx),dzmm(lx),dwat(lx),icesoi_vol(lx)
      real*8 columncapacity,rootfrac(lx),rootdistribution(lx)
      real*8 wfact,fsat,zw,wetness,wetness_integral,sumthickness
      real*8 wetness_integralupper,upper_thickness,subsurfacerunoff
      real*8 densityC1,densityC2,densityC3,soilCdensity,sumfrac
      real*8 fracC4layer,sumjunk1,sumjunk2,sumjunk3,stemp_4layer(sx:lx)
      real*8 production_k,decomp_k,doc4layer(0:lx),DOCsubsurface
      data soil_xnodes/-0.696D0,-0.402D0,-0.292D0,-0.164D0,-0.070D0,
     $    0.000D0,0.01D0,0.03D0,0.08D0,0.13D0,0.23D0,0.33D0,0.45D0,
     $  0.55D0,0.70D0,1.05D0,1.40D0,1.75D0,2.25D0,2.75D0,3.25D0,4.75D0, 
     $  7.25D0,14.0D0,22.0D0,31.0D0,40.0D0,50.0D0,60.00D0/
 
      logical ET_SWITCH,INFILTRATION_SWITCH
      parameter(dtime=3600.0d0,secperday=86400.0d0,wfact=0.3d0)
      parameter(smpmin = -1.0d8,latentheat=7.974474d4)

      if (iwetland .ne. 0) stop 'stopped...a wetland grid'

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

      sum_transpiration = 0.0d0
      subsurfacerunoff = 0.0d0
      watersum4layers = 0.0d0       ! for water balance check
      water_in_NEW = 0.0d0
      water_out_NEW = 0.0d0
      columncapacity = 0.0d0    ! total capacity over all layers. Get with sum.

c  Next line sums water and ice over all soil layers for water balance check
      do i = 0, lx   ! index zero is surface water layer
         watersum4layers = watersum4layers + wat4layerprev(i) + 
     $        ice4layerprev(i)
         DOC4layer(i) = 0.0d0        ! Set to zero. Estimate below.
      enddo
      
      water_in_NEW = watersum4layers + soil_availwater

c  zbottom(i) is depth [mm] of bottom of the layer, from the surface
c  soillayercenter is depth [mm] of center of the layer, from the surface
      ipeatlayers = 0
      do i = 1, lx         
         zmm(i) = soil_xnodes(i) * 1.0d3
      enddo

      sumthickness = 0.0d0    ! to sum the layer thicknesses
      
      call compute_dzmm(soil_xnodes*1.0d3, lx, dzmm)

c      sumfrac = 0.0d0
      do i = 1, lx
         sumthickness = sumthickness + dzmm(i) ! sum the layer thicknesses

c  FLAG WHICH LAYERS ARE PART OF ROOT ZONE, FROM WHICH ET WILL BE DRAWN
c  Also determine root fraction in case it is needed. Estimated 
c  fraction of roots at this depth obtained using function 
c  published in Jackson et al., 1996 "A global analysis of root 
c  distributions for terrestrial biomes", Oecologia Volume 108, Number 3.
c              y = 1 - Beta ^ d, 
c  where Y is the cumulative root fraction with depth (a proportion
c  between 0 and 1), d is soil depth (in cm), and ~ is the fitted
c  parameter. Larger values of ~] imply deeper rooting profiles.

         iroot_flagged(i) = 0 ! initialize to 0, assume not part of rootzone
         rootfrac(i) = 0
         if (soilrootdepth .gt. zmm(i)) then
            iroot_flagged(i) = 1      ! these layers ARE root zone layers
!                                                 exponent is depth in cm
            rootdistribution(i) = 1.0d0 - 0.943d0 ** (zmm(i)/10.0d0) 
            if (i .eq. 1) then
               rootfrac(i) = rootdistribution(i)
               sumfrac = sumfrac + rootfrac(i)
            else
               rootfrac(i) = rootdistribution(i)-rootdistribution(i-1)
               sumfrac = sumfrac + rootfrac(i)
            endif
 104        format(i2,4f10.5)
         endif 
         
c  GET THE PORE SPACE TRHICKNESS capacity4layer(i), BY 
c    MULTIPLING LAYER THICKNESS BY porosity4layer(i).  THEN DETERMINE
c    field_capacity(i) USING fieldcapacity IN %. WILTING POINT WILL BE 
c    DETERMINED BELOW.

         capacity4layer(i) = dzmm(i) * porosity4layer(i)
         field_capacity(i) = dzmm(i) * fieldcapacity(i)
         wiltingPT4layer(i) = dzmm(i) * wiltingpoint

         if (capacity4layer(i) .lt. 0.0d0) stop 'Capacity space < 0'
         columncapacity = columncapacity + capacity4layer(i)

         water4layer(i) = wat4layerprev(i)  ! set today's water 
         ice4layer(i) = ice4layerprev(i)    ! set today's ice

      enddo    ! end of loop to set up configuration of layers
      water4layer(0) = wat4layerprev(0)   ! set todays surface water
      ice4layer(0) = ice4layerprev(0)       ! set todays surface ice
      

c  Add ponded surface to snowmelt, rain.     
      surplus = soil_availwater + water4layer(0)
      water4layer(0) = 0.0d0

      sumbaseflowNEW = 0.0d0             ! zero out for a sum below in code
      watersum4layers = 0.0d0            ! zero out for a sum below in code
      changesurfacelayer = 0.0d0         ! zero out for a sum below in code

c  Infiltration
      if (INFILTRATION_SWITCH .and. ice4layer(1) .eq. 0.0d0) then
         infiltrateMAX=infil_limit *
     $       (1.0d0 - water4layer(1)/capacity4layer(1))
         water4layer(0) =  max((surplus - infiltrateMAX), 0.0d0)
         surplus2soil = surplus - water4layer(0)
      else
         infiltrateMAX = 0.0d0    ! always set to zero for the write statement
         water4layer(0) = surplus   ! use for ponded surface method
         surplus2soil = 0.0d0       
      endif

      surplus2soil = surplus2soil / 24.     ! scale daily value to hourly

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c  Water flow between layers

c  water4layer(i) - todays water mass in layer [mm]
c  ice4layer(i) - ice mass in layer [mm]
c  vol_ice(i) - the volume of ice relative to capacity [%]
c  eff_porosity(i) - porosity after considering ice in layer [%]
c  vol_liq(i) - liquid water volume relative to total capacity [%]
c  icefrac(i) - fraction of ice relative to total water [%]

      do itimestep = 1, 24

      do i = 1, lx
         vol_liq(i) = water4layer(i) / dzmm(i)
      enddo

c  Hydraulic conductivity and soil matric potential and their derivatives

      sdamp = 0.0d0
      do i = 1, lx       

         s1 = 0.5d0*(porosity4layer(i)+porosity4layer(min(lx,i+1)))
         s2 = 0.5d0*(vol_liq(i) + vol_liq(min(lx,i+1)))/s1
         s2 = min(1.0d0,s2)
       
         if (ice4layer(i).eq.0.0d0) then
            hk(i) = hk_sat(i)*s2**beta(i)  
            dhkdw(i) = hk(i)*beta(i)/s2/s1*0.5d0 
         else
            hk(i) = 0.0d-3
            if(i>1) hk(i-1) = 0.0d-3
            dhkdw(i) = 0.0d0
            if(i>1) dhkdw(i-1) = 0.0d0
         endif
          

         s_node = max(vol_liq(i)/porosity4layer(i),0.01d0) 
         s_node = min(1.0d0,s_node)

         smp(i) = psi_sat(i)*s_node**(-beta(i)) 
         dsmpdw(i) = -beta(i)*smp(i)/(s_node*porosity4layer(i)) 

         if (ice4layer(i) .ne. 0.0d0) then     
            smp(i) = 0.0d0             ! if layer has has ice, set             
            dsmpdw(i) = 0.0d0          ! matrix potential to zero 
         endif 
         
       enddo   ! end of a soil layer loop setting up K and matrix potential

c   Set up r, a, b, and c vectors for tridiagonal solution

! Node j=1 (top) in CLM   

!dir$ concurrent
!cdir nodep
      do i = 1, 1  
         qin    = surplus2soil/dtime      ! resume edits here
         den    = (zmm(i+1)-zmm(i))
         num    = (smp(i+1)-smp(i)) - den
         qout   = -hk(i)*num/den
         dqodw1 = -(-hk(i)*dsmpdw(i) + num*dhkdw(i))/den
         dqodw2 = -( hk(i)*dsmpdw(i+1) + num*dhkdw(i))/den
         rmx(i) =  qin - qout
         amx(i) =  0.0d0
         bmx(i) =  dzmm(i)*(sdamp+1.0d0/dtime) + dqodw1
         cmx(i) =  dqodw2
            
      enddo 

c  Here for layers between 2 and down to next to last one (lx-1)
      do i = 2, lx - 1

         den    = (zmm(i) - zmm(i-1))
         num    = (smp(i)-smp(i-1)) - den
         qin    = -hk(i-1)*num/den
         dqidw0 = -(-hk(i-1)*dsmpdw(i-1) + num*dhkdw(i-1))/den
         dqidw1 = -( hk(i-1)*dsmpdw(i)   + num*dhkdw(i-1))/den

         den    = (zmm(i+1)-zmm(i))
         num    = (smp(i+1)-smp(i)) - den
         qout   = -hk(i)*num/den
         dqodw1 = -(-hk(i)*dsmpdw(i)   + num*dhkdw(i))/den
         dqodw2 = -( hk(i)*dsmpdw(i+1) + num*dhkdw(i))/den

         rmx(i) =  qin - qout
         amx(i) = -dqidw0
         bmx(i) =  dzmm(i)/dtime - dqidw1 + dqodw1
         cmx(i) =  dqodw2

      end do

!  This section is for bottom layer 
      do i = lx, lx
         den    = (zmm(i) - zmm(i-1))
         num    = (smp(i)-smp(i-1)) - den
         qin    = -hk(i-1)*num/den
         dqidw0 = -(-hk(i-1)*dsmpdw(i-1) + num*dhkdw(i-1))/den
         dqidw1 = -( hk(i-1)*dsmpdw(i)   + num*dhkdw(i-1))/den
         qout   =  0.0d0        ! zero-flow bottom boundary condition
         dqodw1 =  0.0d0        ! zero-flow bottom boundary condition
         rmx(i) =  qin - qout
         amx(i) = -dqidw0
         bmx(i) =  dzmm(i)/dtime - dqidw1 + dqodw1
         cmx(i) =  0.0d0

      end do

      call Tridiagonal(1,lx,amx,bmx,cmx,rmx,dwat)

    ! Renew the mass of liquid water
c  Next block of code removes or adds water from each layer based on solution
c  of Richard's equation and the tridiagonal matrix.

      do i = 1, lx
         water4layer(i) = water4layer(i) + dwat(i)*dzmm(i)   

      enddo

      enddo    ! end of outer loop of time step 

c  This block loops from bottom layer upward, and sends excess water to above
c  layer if the lower layer goes above capacity. 
      excess = 0.0d0
      excess2 = 0.0d0
      do i = lx, 2, -1
         if (ice4layer(i).eq. 0.0d0) then
            excess = max(water4layer(i)-capacity4layer(i),0.0d0)
            water4layer(i) = min(water4layer(i),capacity4layer(i))
            if(ice4layer(i-1) .eq. 0.0d0) then
               water4layer(i-1) = water4layer(i-1) + excess
            else   
               sumbaseflowNEW = sumbaseflowNEW + excess
            endif
         endif
      enddo         ! next block handles first layer
      if (ice4layer(1).eq. 0.0d0) then
         excess = max(water4layer(1)-capacity4layer(1),0.0d0)
         water4layer(1) = min(water4layer(1),capacity4layer(1))
         water4layer(0) = water4layer(0) + excess  ! send excess above surface
      endif   

c    ccccccccccc    Baseflow done over next couple of blocks   ccccccccccccc
c  Generate baseflow if layer is over pore capacity (or layer field capacity).
      excess = 0.0d0
      
      do i = 1, lx
         baseflow_array(i) = 0.0d0          
         if (ice4layer(i).eq.0.0d0) then
            excess = max(water4layer(i)-field_capacity(i),0.0d0)
            base_flow = excess * deepbaseflowfactor
            water4layer(i) = water4layer(i) - base_flow
            sumbaseflowNEW = sumbaseflowNEW + base_flow
            baseflow_array(i) = base_flow   
         endif
 
      enddo  

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     DOC estimation test code. January 2020.
c
      iflag = 0

      call docleaching(iWinter,soil_xnodes,peatfrac,stemp_4layer,
     $  water4layer,capacity4layer,baseflow_array,densityC1,
     $  densityC2,densityC3,soilDOC,production_k,decomp_k,
     $  DOC4layer,DOCsubsurface,iwrite)

      if (sumbaseflowNEW .gt. 0.0d0 .and. DOCsubsurface .eq. 0.0d0) then
            DOCsubsurface = -888.8
      else if (sumbaseflowNEW .eq. 0.0d0) then
         DOCsubsurface = -999.9
      endif

      if (iwrite .eq. 1) then
      endif   
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccc     INFILTRATION AND BASEFLOW NOW COMPLETED   cccc
cccc      NEXT SECTION HANDLES EVAPOTRANSPIRATION  cccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c                      cccccccccccccccccccccccccccccccccc

c  Here set up calculation to estimate ET from each layer

      if (airtemp .le. 0.0d0) then ! no transpiration when air temp < 0
         sum_transpiration = 0.0d0
         goto 2
      endif

      wfractionSUM = 0.0d0   ! sum to do weighting of water fraction each depth

      do i = 1, lx
         if (water4layer(i) .le. wiltingPT4layer(i) .or.
     $            wiltingPT4layer(i) .lt. 1.0d-2 .or. 
     $                       ice4layer(i) .gt. 0.0d0) then
            wfractionNEW(i) = 0.0d0   ! no ET will occur
         else

c  The next lines do retention fraction as a function of available space
c  which is in the denominator. Fraction is 1 if water is at capacity. It
c  is zero if water is at or below the wilting point. 
            wfractionNEW(i) = ((water4layer(i) - wiltingPT4layer(i)) /
     $           (capacity4layer(i) - wiltingPT4layer(i)))
            wfractionNEW(i) = dmin1(wfractionNEW(i),1.0d0)
            wfractionNEW(i) = dmax1(wfractionNEW(i),0.0d0)
         endif
         
c  Sum up the wilting point fraction (wfractionNEW) over the root zone layers
         if (iroot_flagged(i) .eq. 1) then
            wfractionSUM = wfractionSUM + wfractionNEW(i) * rootfrac(i)
         endif
      enddo     ! end of loop over soil layers to get wilting point parameters

      if (ivegclass .eq. 4 .or. ivegclass .eq. 5 .or.
     $     ivegclass .eq. 8) then
         surface_conduct_factor = param3
      else
         surface_conduct_factor = param4
      endif   
      
c  Now pull ET from each layer in root zone 
      do i = 1, lx

         if (iroot_flagged(i) .eq. 0) goto 2 

         if (wfractionSUM .ne. 0.0d0 .and. ET_SWITCH) then
            if (soil_availwater .lt. soil_pet) then
               trans4layer = soil_pet * rootfrac(i) * wfractionNEW(i)
               trans4layer = trans4layer * surface_conduct_factor
            else
               trans4layer = soil_pet *
     $              ((rootfrac(i) * wfractionNEW(i)) / wfractionSUM)
            endif   
         else
            trans4layer = 0.0d0
         endif

         trans4layer =min(trans4layer,water4layer(i)-wiltingPT4layer(i))
         trans4layer = max(trans4layer,0.0d0)
         if (trans4layer .gt. water4layer(i)) 
     $                        trans4layer = water4layer(i) ! extra check
         water4layer(i) = water4layer(i) - trans4layer
         sum_transpiration = sum_transpiration + trans4layer
         if (ice4layer(i) .gt. 0.0d0 .and. trans4layer .gt. 0.0d0) then
            print*, 'STOP -- layer has ice and transpiration > 0 '
            stop
         endif
         
       enddo    ! end of transpiration loop over the soil layers

       if (sum_transpiration .gt. soil_pet) stop 'Accum. ET > PET'

 2    continue

c                   -----------------------------------
c  Check to ensure that no layer has more water+ice than its capacity.
      do i = 1, lx
         sumjunk1 = water4layer(i) + ice4layer(i)
         if (sumjunk1 .gt. capacity4layer(i)) then
            if (water4layer(i) .gt. 0.0d0) then
               excess = max(water4layer(i)-capacity4layer(i),0.0d0)
               water4layer(i) = capacity4layer(i)
               sumbaseflowNEW = sumbaseflowNEW + excess
            else
               excess = max(ice4layer(i)-capacity4layer(i),0.0d0)
               ice4layer(i) = capacity4layer(i)
               sumbaseflowNEW = sumbaseflowNEW + excess
            endif 
            write(*,'(a)') 
     $       'Layer water+ice > capacity. Sending extra to runoff'
            write(*,'(3i5,4f8.1)')
     $           nsimdays, icell, i, capacity4layer(i),
     $            water4layer(i), ice4layer(i), excess
            if (ice4layer(i) .gt. capacity4layer(i) .and. 
     $                 nsimdays .gt. 30) then
               write(*,'(a)') 
     $      'ERROR: Layer ice > capacity for too many days.'
               stop
            endif
         endif
      enddo

c  Sum current water in soil layers so can do water balance check.
      do i = 0, lx  ! index zero is the surface water layer
         watersum4layers = watersum4layers + 
     $                             ( water4layer(i) + ice4layer(i) )
      enddo
      
c  Water balance check
      water_out_NEW = watersum4layers + sumbaseflowNEW + 
     $      subsurfacerunoff + soiloverlandROnew + sum_transpiration
      checkNEW = water_out_NEW - water_in_NEW
 
      if (abs(checkNEW) .gt. 1.0d-6) then    
         print*, 'water balance not maintained in soil routine'
         print*, icell, checkNEW, water_in_NEW, water_out_NEW,  
     $   watersum4layers, soil_availwater, sumbaseflowNEW,
     $    soiloverlandROnew, subsurfacerunoff, sum_transpiration 
         stop
      endif   

      return
      end
