C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#          *****  PWBM (v4) function: surfaceConduct  ****           ##
C#    Function returns approximation for surface conductance          ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
c
c     Function is a simplification of algorithms described in
c        Mu_MOD16_ET_ATBD2013.pdf
c     pages 10-13.  Equations 24 and 25 most relevant.         
c
      real*8 function surfaceConduct (tair,rLAI,iwrite)

      implicit none
      integer iwrite
      real*8 rLAI,cl,cs,tair,tmin,tminopen,tminclose,tmin_multiplier
      real*8 surfaceConduct
      parameter (cl=6.5d-3,tminopen=9.28d0,tminclose=-8.0d0)

      tmin = tair - 5.5d0
      if (tmin .ge. tminopen) then
         tmin_multiplier = 1.0d0
      else if (tmin .le. tminclose) then
         tmin_multiplier = 0.0d0
      else
         tmin_multiplier = (tmin - tminclose) / (tminopen - tminclose)
      endif

      cs = cl * tmin_multiplier
      surfaceConduct = cs * rLAI
      if (iwrite .eq. 1) then
c         write(92,3) tair, tmin_multiplier, rLAI, cl, cs, surfaceConduct
c         write(92,3) rLAI, surfaceConduct
      endif   
c      surfaceConduct = 
c 3    format(6f12.4)
 3    format(2f12.6)
      
      return
      end
