C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: Tridiagonal  *****       ##
C#    Subroutine solves tridiagonal matrix                            ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************

      subroutine Tridiagonal (lbj, ubj, a, b, c, r, u)
!
! !DESCRIPTION:
! Tridiagonal matrix solution
! !ARGUMENTS:
      implicit none
      integer j             ! loop index
      integer lbj, ubj      ! lbinning and ubing level indices
      real*8 a(ubj)         ! "a" left off diagonal of tridiagonal matrix
      real*8 b(ubj)         ! "b" diagonal column for tridiagonal matrix
      real*8 c(ubj)         ! "c" right off diagonal tridiagonal matrix
      real*8 r(ubj)         ! "r" forcing term of tridiagonal matrix
      real*8 u(ubj)         ! solution
      real*8 bet,gam(ubj)   ! local variables

      bet = b(1)

!  for the PWBM soil code, lbj is 1 and ubj is 23
      do j = lbj, ubj
!       if (j == jtop(ci)) then
         if (j == lbj) then
            u(j) = r(j) / bet
         else
            gam(j) = c(j-1) / bet
            bet = b(j) - a(j) * gam(j)
            u(j) = (r(j) - a(j)*u(j-1)) / bet
         endif
      enddo

      do j = ubj-1,lbj,-1
         u(j) = u(j) - gam(j+1) * u(j+1)
      enddo


      return
      end 
