C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: read_LAI  *****          ##
C#  Subroutine returns daily LAI by interpolating from monthly values ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************
      
      subroutine read_LAI(fname,jyr,ngrid,valnew)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c  THIS ROUTINE INPUTS: FILENAME, YEAR, NUMBER OF GRIDS
c  AND RETURNS ARRAY OF VALUES BY MONTH, DAY AND GRID.
c  IT USES CRESSMANS ALGORITHM TO INTERPOLATION BETWEEN MONTHLY VALUES.
c  VALUES FOR FIRST HALF OF JANUARY AND LAST HALF OF DECEMBER ARE STATIC AT
c  THOSE MONTHLY VALUES. ALL OTHER DAILY VALUES ARE INTERPOLATED 
c  BETWEEN THE MONTLY VALUES INPUT TO THE ROUTINE.
    
      implicit none
      integer m,k,iday,igrid,ngrid,imon,jyr,ndays,juldate
      integer idayofyear
      real rlat,rlon,gridpt
      real valnew(12,31,ngrid),xday(12),dat1(ngrid),dat2(ngrid),ynew
      real datvals(12,ngrid),x1,x2,data1,data2
      data xday /15.,46.,74.,105.,135.,166.,196.,227.,258.,288.,
     $         319.,349./
      character fname*100,header*60

      open(10,file=fname)
      read(10,'(a60)') header
      read(10,'(a60)') header
      do m=1, ngrid
         read(10,*) rlat, rlon, (datvals(k,m), k=1, 12)
      enddo

c  Begin month, day, and grid loops
         do imon = 1, 12
            call days4month(jyr,imon,ndays)
c  Fill arrays with missing values
            do igrid = 1, ngrid
               do iday = 1, 31
                  valnew(imon,iday,igrid) = -9999.9
               enddo
            enddo

            do iday = 1, ndays
               juldate = idayofyear(imon,iday,jyr)
               gridpt = real(juldate)
               do igrid = 1, ngrid

                  if (imon .eq. 1 .and. iday .le. 14) then
                     valnew(1,iday,igrid) = datvals(1,igrid)
                     goto 200
                  else if (imon .eq. 12 .and. iday .ge. 15) then
                     valnew(12,iday,igrid) = datvals(12,igrid)
                     goto 200
                  endif
                  if (iday .eq. 15) then
                     x1 = xday(imon)
                     x2 = xday(imon+1)
                     dat1(igrid) = datvals(imon,igrid)
                     dat2(igrid) = datvals(imon+1,igrid)
                  endif
                  data1 = real(dat1(igrid))
                  data2 = real(dat2(igrid))
                  call cressmanpt(x1,x2,data1,data2,gridpt,ynew)
                  valnew(imon,iday,igrid) = ynew 
 200           enddo          ! grid loop
            enddo      ! day loop
         enddo   ! month loop

      close(10)   
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine cressmanpt(xx1,xx2,dataval1,dataval2,gridx,ynew)

      real xx1,xx2,dataval1,dataval2,gridx,ynew

      r = (xx2 - xx1) ** 2
      d1 = abs(gridx - xx1) ** 2
      d2 = abs(gridx - xx2) ** 2
      weight1 = (r - d1)/(r + d1)
      weight2 = (r - d2)/(r + d2)

      yhat1 = dataval1 * weight1
      yhat2 = dataval2 * weight2
      sumweight = weight1 + weight2
      ynew = (yhat1 + yhat2) / sumweight

      return
      end


