C#######################################################################
C#######################################################################
C#                                                                    ##
C#               PPPP   W           W  BBBBBB   M       M             ##
C#               P   P   W         W   B    B   MM     MM             ##
C#               PPPP     W   W   W    BBBBBB   M M   M M             ##
C#               P         W W W W     B    B   M  M M  M             ##
C#               P          W   W      BBBBBB   M   M   M             ##
C#                                                                    ##
C#                                                                    ##
C#                    PERMAFROST WATER BALANCE MODEL                  ##
C#                     +++++   Version 4.0   +++++                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#                 PWBM VERSION 4.0 CAN BE OBTAINED AT:               ##
C#                                                                    ##
C#                  https://blogs.umass.edu/csrc/pwbm                 ##
C#                                                                    ##
C# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .##
C#                                                                    ##
C#              *****  PWBM (v4) subroutine: get_date  *****          ##
C#    Subroutine gets month and day and reads/writes data             ##
C#                                                                    ##
C#                                                                    ##
C#     NOTE:   See notes in PWBM main routine, file pwbmModel_v4.f    ##
C#                                                                    ##
c***********************************************************************

      subroutine get_date(iyr, julday, imonth, iday)
c     subroutine gets the month and the day of the month from Julian 
c     day 
c     inputs:   iyr    -  the year
c               julday -  the Julian day
c     outputs:  imonth -  the month
c               iday   -  the day of the month 

      implicit none
      integer  iyr,julday,imonth,iday,imth1,sumday
      integer  imth, iday0, ileap
      integer  monlastday(12), imonday2(12) 
      data     monlastday /31,28,31,30,31,30,31,31,30,31,30,31/

c     leap year or not
      ileap = 0
      if(mod(iyr,4).eq.0 .and. mod(iyr,100).ne.0) ileap = 1
      if(mod(iyr,400) .eq. 0) ileap = 1
      if(ileap .eq. 1) then
         monlastday(2) = 29
      else
         monlastday(2) = 28
      endif

c     the last julian day of each month
      do imth=1,12
         sumday = 0
         do imth1=1,imth
            sumday = sumday + monlastday(imth1)
         enddo
         imonday2(imth) = sumday
c        print *, imonday2(imth)
      enddo             

c     then find the month and day of that month
      do imth=1, 12
         if( julday .le. imonday2(imth)) exit
      enddo
      imonth = imth   
      
      if(imonth .eq. 1) then
         iday0 = julday 
      else
         iday0 = julday - imonday2(imth-1)
      endif
      iday = iday0

      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine write_daily_data(fpath,rmissing,dataname,iyr,imon,
     $                             ncells,dataval,rlon,rlat)

c    subroutine writes daily outputs of PWBM for a specific year and month

      implicit none
   
      integer   iyr,imon,iday,icell,ncells,narray,nn,ileap
      integer   length, length1
      parameter (narray=579)
      integer   ntotdays,monlastday(12)
      real*8    dataval(narray,31), mthval(narray,31), rmissing
      real      rlat(narray), rlon(narray)
      character datestring2*11, str2*200, str1*200
      character fname*100, fpath*100, dataname*40

      str2='"CellID" "Lat" "Long" "01" "02" "03" "04" "05" "06" "07" "08
     $" "09" "10" "11" "12" "13" "14" "15" "16" "17" "18" "19" "20" "21"
     $ "22" "23" "24" "25" "26" "27" "28" "29" "30" "31"'
 121  format(i5,2f10.4,31f8.1)

      length = index(fpath, ' ') - 1 
      write(datestring2, '(i4,''.'',i2.2,''.txt'')') iyr, imon
c         write(datestring2, '(i4,''.'',i2.2,''.dat'')') iyr, imon 
      fname=fpath(1:length)//datestring2
      open(32,file=fname)
        
      write(str1,'(i4,a,a)') iyr, ' ', dataname
      length1 = index(str1, ';') - 1
      write(32,'(a)') str1(1:length1)
      write(32,'(a180)') str2
      do icell = 1, ncells
         if (rlon(icell) .lt. 0.0) 
     $        rlon(icell) = 180. + (180. + rlon(icell))
         write(32,121) icell, rlat(icell), rlon(icell),
     $        (dataval(icell,nn), nn=1,31)
      enddo
      close(32)  
 
      return
      end 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine write_daily_data2(fpath,rmissing,dataname,iyr,imon,
     $                             ncells,dataval,rlon,rlat)

c    subroutine writes daily outputs of PWBM for a specific year and month
c
c     This subroutine differs from write_daily_data in that this one
c     writes out data as f12.4
      
      implicit none
   
      integer   iyr,imon,iday,icell,ncells,narray,nn,ileap
      integer   length, length1
      parameter (narray=579)
      integer   ntotdays,monlastday(12)
      real*8    dataval(narray,31), mthval(narray,31), rmissing
      real      rlat(narray), rlon(narray)
      character datestring2*11, str2*200, str1*200
      character fname*100, fpath*100, dataname*40

      str2='"CellID" "Lat" "Long" "01" "02" "03" "04" "05" "06" "07" "08
     $" "09" "10" "11" "12" "13" "14" "15" "16" "17" "18" "19" "20" "21"
     $ "22" "23" "24" "25" "26" "27" "28" "29" "30" "31"'
 121  format(i5,2f10.4,31f8.1)
 122  format(i5,2f10.4,31f12.4)

      length = index(fpath, ' ') - 1 
      write(datestring2, '(i4,''.'',i2.2,''.txt'')') iyr, imon
c         write(datestring2, '(i4,''.'',i2.2,''.dat'')') iyr, imon 
      fname=fpath(1:length)//datestring2
      open(32,file=fname)
        
      write(str1,'(i4,a,a)') iyr, ' ', dataname
      length1 = index(str1, ';') - 1
      write(32,'(a)') str1(1:length1)
      write(32,'(a180)') str2
      do icell = 1, ncells
         if (rlon(icell) .lt. 0.0) 
     $        rlon(icell) = 180. + (180. + rlon(icell))
         write(32,122) icell, rlat(icell), rlon(icell),
     $        (dataval(icell,nn), nn=1,31)
      enddo
      close(32)  
 
      return
      end 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
      subroutine write_mth_data(fname,dataname,iyr,ncells,dataval,
     $           rlon,rlat)
c      subroutine writes monthly outputs of PWBM for a specific year
       implicit none
       
       integer    narray, iyr, icell, nn, ncells
       parameter (narray=579)
       real*8     dataval(narray, 12)
       real       rlat(narray), rlon(narray)  
       character  fname*100, dataname*40

       open(90,file=fname)
       write(90,'(i4,a)') iyr, dataname//' (i6, 2f10.4, 12f8.1)'
       write(90,'(a)') '"CellID" "Lat" "Long" "01" "02" "03" "04" "05" 
     $"06" "07" "08" "09" "10" "11" "12"'
       do icell = 1, ncells
          if (rlon(icell) .lt. 0.0) 
     $        rlon(icell) = 180. + (180. + rlon(icell))
          write(90,118) icell, rlat(icell), rlon(icell),
     $       (dataval(icell,nn), nn=1,12)
118    format(i5,2f10.4,12f8.1)
       enddo
       close(90)
      end 


      subroutine read_data_ann(fpath, rmiss, iyr, dataval)
c
c  Subroutine opens the specified input file and reads data into an array.
c  Returns array to main program.  The array (time,j,i) is a continuous
c  record of valid values in the file.  i.e., Missing values representing
c  padded days 31, 30, or 29 at end of a month have been screened out.
c  This version inputs multiple files (one year) and returns the 365 or 366
c  values to main program.
c  Note: format must be set in this subroutine
c
c  Inputs:  fpath    - Character string designating path and file prefix name for input
c           rmiss    - Missing vaue code.  Values will be screened out
c            iyr     -  the year of data to be read
c  Outputs: dataval  - Array of data values returned to main program
c                   Dimensions are (time, lat index, lon index)

      implicit none
      integer i,j,k,m,nfiles,igrid,ifrstday,nvals
      integer iyr,imth,ileap,length,imth1,sumday  
      integer monlastday(12), imonday1(12)
      parameter (nvals=579)
      real rlat,rlon
      real*8 dataval(366,nvals),rmiss,dat(31)
      character fname*100,fpath*100,fmt*19,header*60
      character datestring*7
      data monlastday /31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/

c     leap year or not
      ileap = 0
      if (mod(iyr, 4).eq.0 .and. mod(iyr,100).ne.0) 
     $     ileap = 1
      if (mod(iyr, 400).eq.0) ileap = 1
      if (ileap .eq. 1) then
          monlastday(2) = 29
      else
         monlastday(2) = 28
      endif

c     the initial julian day of each month
      imonday1(1) = 0
      do imth=2,12
          sumday = 0
          do imth1=1,imth-1
             sumday = sumday + monlastday(imth1)
          enddo
          imonday1(imth) = sumday
      enddo

c     read monthly files and then put data together
      do imth= 1, 12
          length = index(fpath, ' ') - 1
          write(datestring, '(i4,''.'',i2.2)') iyr, imth
          fname = fpath(1:length)//datestring//'.txt' 
         
          open(10,file=fname)
c          read(10,'(a60)') header
c          read(10,'(a60)') header
          write(fmt,'(''(24x,31f8.1)'')')   !set format here
          do m=1, nvals
c            read(10,fmt) (dat(k), k=1,31)
c             read(10,*) igrid, rlat, rlon, (dat(k), k=1,31)
             read(10,*) (dat(k), k=1,31)
c            print*, m, dat(1), dat(31)
             
             do k=1, monlastday(imth)                  
                 dataval(imonday1(imth)+k, m) = dat(k)
             enddo
          enddo
          close(10)
      enddo

      return
      end

      subroutine read_data(fname,rmiss,idata,dataval)
c
c  Subroutine opens the specified input file and reads data into an array.  
c  Returns array to main program.  The array (time,j,i) is a continuous
c  record of valid values in the file.  i.e., Missing values representing
c  padded days 31, 30, or 29 at end of a month have been screened out.
c  This version inputs one file (one month) and returns the 28, 29, 30, or 31
c  values to main program.
c  Note: format must be set in this subroutine
c
c  Inputs:  fname - Character string designating path and file name for input
c           rmiss - Missing vaue code.  Values will be screened out
c  Outputs: idata - Number of non-missing data returned to main program
c           data  - Array of data values returned to main program
c                   Dimensions are (time, lat index, lon index)

      implicit none
      integer i,j,k,m,nfiles,igrid,ifrstday,idata,nvals
c      parameter (nvals=39926)
      parameter (nvals=579)
      real rlat,rlon
      real*8 dataval(31,nvals),rmiss,dat(31)
      character fname*100,fmt*19,header*60

      open(10,file=fname)
c      read(10,'(a60)') header
c      read(10,'(a60)') header
      write(fmt,'(''(24x,31f8.1)'')')   !set format here
      do m=1, nvals
c         read(10,*) igrid, rlat, rlon, (dat(k), k=1,31)
         read(10,*) (dat(k), k=1,31)
         idata = 0
         do k=1, 31
            dat(k) = dat(k)
            idata=idata+1   
            dataval(idata,m) = dat(k)
         enddo
      enddo

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine read_NPP(fpath,iyr,rmiss,datavalue)
c
c  Subroutine opens the specified input file and reads data into an array.  
c  Returns array to main program.  The array (time,j,i) is a continuous
c  record of valid values in the file.  i.e., Missing values representing
c  padded days 31, 30, or 29 at end of a month have been screened out.
c  This version inputs one file (one month) and returns the 28, 29, 30, or 31
c  values to main program.
c  Note: format must be set in this subroutine
c
c  Inputs:
c     fname - Character string designating path and file name for input
c     rmiss - Missing vaue code.  Values will be screened out
c     iyr - year of data      
c  Outputs: 
c        datavalue  - Array of data values returned to main program
c      

      implicit none
      integer i,j,k,m,iyr,length,nfiles,igrid,idata,nvals
      parameter (nvals=579)
c      parameter (nvals=39926)
      real rlat,rlon
      real*8 datavalue(nvals),rmiss,dat_val
      character fpath*100,fname*100,fmt*19,header*60,datestring*10

      length = index(fpath, ' ') - 1 
      write(datestring, '(i4,''.ascii'')') iyr
c         write(datestring2, '(i4,''.'',i2.2,''.dat'')') iyr, imon 
      fname=fpath(1:length)//datestring
c      print*, fname
c      stop
      open(10,file=fname)
      read(10,'(a60)') header
c      write(fmt,'(''(24x,31f8.1)'')')   !set format here
      idata = 0
      do m=1, nvals
         read(10,*) igrid, rlat, rlon, dat_val
         datavalue(m) = dat_val
         if (dat_val .eq. rmiss) idata = idata + 1
      enddo

      return
      end
